@@ -254,7 +254,7 @@ class cognet:
                 sample,
                 steps,
                 immutable=False):
-        '''perturb the sample based on thet qnet distributions and number of steps
+        '''perturb the sample based on the qnet distributions and number of steps
 
         Args:
           sample (1d array-like): sample vector, must have the same num of features as the qnet
@@ -271,12 +271,14 @@ class cognet:
             return qsample(sample,self.qnet,steps)
 
     def random_sample(self,
-                      df=None):
+                      df=None,
+                      n=1):
         '''compute a random sample from the underlying distributions of the dataset, by column.
         
         
         Args:
           df (pandas.DataFrame): Desired data to take random sample of. Defaults to None, in which case qnet samples are used.
+          n (int): number of random samples to take. Defaults to 1.
           
         Returns:
           return_df (pd.DataFrame): Random sample drawn from underlying distribution of each column.
@@ -290,7 +292,7 @@ class cognet:
         # take random sample from each of the columns based on their distribution
         return_df = pd.DataFrame()
         for col in samples_.columns:
-            return_df[col] = samples_[col].sample(n=1)
+            return_df[col] = samples_[col].sample(n=n, replace=True).values
             
         return return_df
     
@@ -460,7 +462,7 @@ class cognet:
     def distfunc_line(self,
                     i,
                     return_dict=None):
-        '''compute the distance for a single sample from all other samples
+        '''compute the distance for a single sample against all other samples
 
         Args:
           i (int): row
@@ -776,21 +778,26 @@ class cognet:
             raise ValueError("load_data first!")
 
     def dissonance(self,
-                sample_index,
-                return_dict=None,
-                MISSING_VAL=0.0):
+                    sample_index=0,
+                    return_dict=None,
+                    MISSING_VAL=0.0,
+                    sample=None):
         '''compute dissonance for a single sample, helper function for all_dissonance
         
         Args:
-          sample_index (int): index of the sample to compute dissonance
+          sample_index (int): index of the sample to compute dissonance. Defaults to 0.
           return_dict (dict): dictionary containing multiprocessing results
           MISSING_VAL (float): default dissonance value
+          sample (1D array): sample to compute dissonance of, instead of using sample index. Defaults to None.
           
         Returns: 
           diss[self.polar_indices]: ndarray containing dissonance for sample
         '''
         if all(x is not None for x in [self.samples, self.features]):
-            s = self.samples_as_strings[sample_index]
+            if sample is None:
+                s = self.samples_as_strings[sample_index]
+            else:
+                s = sample
             if self.polar_indices is None:
                 self.polar_indices = range(len(s))
 
@@ -905,8 +912,10 @@ class cognet:
                 random_sample[np.where(
                     self.cols==i)[0][0]]=self.__choose_one(
                         self.D_null[np.where(self.cols==i)[0][0]].keys())
+                    
                 rnd_match_prob=np.append(rnd_match_prob,1/len(
                     self.D_null[np.where(self.cols==i)[0][0]].keys()))
+                
                 max_match_prob=np.append(
                     max_match_prob,np.max(
                         list(D[np.where(
@@ -931,7 +940,7 @@ class cognet:
             raise ValueError("load_data first!")
 
     def randomMaskReconstruction(self,
-                                index,
+                                index=None,
                                 return_dict=None,
                                 sample=None,
                                 index_colname="feature_names",
@@ -939,7 +948,8 @@ class cognet:
                                 file_name="recon_tmp.csv",
                                 mask_prob=0.5,
                                 allow_all_mutable=False,
-                                save_samples=False):
+                                save_samples=False,
+                                save_output=True):
         """reconstruct the masked sample by qsampling and comparing to original
         set self.mask_prob and self.steps if needed
 
@@ -952,13 +962,15 @@ class cognet:
           file_name (str): base file name for output files Defaults to "recon_tmp.csv".
           mask_prob (float): float btwn 0 and 1, prob to mask element of sample. Defaults to 0.5
           allow_all_mutable (bool): whether or not all variables are mutable. Defaults to False.
+          save_samples (bool): whether to include sample vectors in the savefile. Defaults to False.
+          save_output (bool): whether or not to save output df to file. Defaults to True.
 
         Raises:
           ValueError: Neither sample or index were given
           ValueError: Both sample and index were given
           
         Returns:
-          return_dict[index]:(1 - (dqestim/dactual))*100,
+          return_values:(1 - (dqestim/dactual))*100,
                             rmatch_u,
                             rmatch,
                             s,
@@ -971,7 +983,7 @@ class cognet:
         elif all(x is not None for x in [sample, index]):
             raise ValueError("Must input either sample or index not both!")
         elif sample is not None:
-            s=np.array(pd.DataFrame(sample).fillna('').values.astype(str)[:])
+            s=sample#np.array(pd.DataFrame(sample).fillna('').values.astype(str)[:])
         elif index is not None:
             s=self.samples_as_strings[index]
         
@@ -988,58 +1000,66 @@ class cognet:
         if not os.path.exists(output_dir):
             os.mkdir(output_dir)
 
-        # qsample sample and calculate distances between original and qsampled 
+        # qsample sample and calculate distances between original vs qsampled and masked
         qs=qsample(s1,self.qnet,self.steps,bp)
         dqestim=qdistance(s,qs,self.qnet,self.qnet)
-        dactual=qdistance(s,s1,self.qnet,self.qnet)
+        dmask=qdistance(s,s1,self.qnet,self.qnet)
         
-        # format and save sample and qsample statistics and values
+        # format and save sample, qsample statistics and values
         cmpf=pd.DataFrame([s,qs,random_sample],
                           columns=self.cols,
                           index=['sample','qsampled','random_sample'])[mask_].transpose()
         cmpf.index.name= index_colname
-        file_name = file_name.replace("tmp", str(index))
-        cmpf.to_csv(output_dir+file_name)
+        if save_output:
+            file_name = file_name.replace("tmp", str(index))
+            cmpf.to_csv(output_dir+file_name)
+            
+        if save_samples:
+            return_values = (1 - (dqestim/dmask))*100,rmatch_u,rmatch,mask_,s,qs,random_sample
+        else:
+            return_values = (1 - (dqestim/dmask))*100,rmatch_u,rmatch,mask_
+        
         if return_dict is not None:
-            if save_samples:
-                return_dict[index] = (1 - (dqestim/dactual))*100,rmatch_u,rmatch,s,qs,random_sample,mask_
-            else:
-                return_dict[index] = (1 - (dqestim/dactual))*100,rmatch_u,rmatch,mask_
-        return return_dict[index]
+            return_dict[index] = return_values
+            return return_dict[index]
+        return return_values
 
     def randomMaskReconstruction_multiple(self,
                                           outfile,
                                           processes=6,
-                                          save_samples=False,
                                           index_colname="feature_names",
                                           output_dir="recon_results/",
                                           file_name="recon_tmp.csv",
                                           mask_prob=0.5,
-                                          allow_all_mutable=False):
+                                          allow_all_mutable=False,
+                                          save_samples=False,
+                                          save_output=True):
         '''runs and saves the results of the predicted masked sample
 
         Args:
           output_file (str): directory and/or file for output.
           processes (int): max number of processes. Defaults to 6.
-          save_samples (boolean): whether or not to save the generated qsamples, random samples, etc. Defaults to False.
           index_colname="feature_names",
           output_dir="recon_results/",
           file_name="recon_tmp.csv",
           mask_prob (float): float btwn 0 and 1, prob to mask element of sample. Defaults to 0.5
           allow_all_mutable (bool): whether or not all variables are mutable. Defaults to False.
+          save_samples (boolean): whether or not to save the generated qsamples, random samples, etc. Defaults to False.
+          save_output (bool): whether or not to save output df to file. Defaults to True.
           
         Returns:
           result: pandas.DataFrame containing masking and reconstruction results.
         '''
         # set columns for mp_compute
         if save_samples:
-            cols = ['rederr','r_prob','rand_err','sample','qsampled','random_sample','mask_']
+            cols = ['rederr','r_prob','rand_err','mask_','sample','qsampled','random_sample']
         else:
             cols = ['rederr','r_prob','rand_err','mask_']
         
         # 
         args=[None, index_colname, output_dir,
-              file_name, mask_prob, allow_all_mutable]
+              file_name, mask_prob, allow_all_mutable, 
+              save_samples, save_output]
         
         result = self.mp_compute(processes,
                                     self.randomMaskReconstruction,
